<?php
namespace Teknisa\CADCTB\Controllers;

use Teknisa\CADCTB\Services\Ger06300CentroDeCustoService;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;

class Ger06300CentroDeCustoController extends TeknisaCrud {

    protected $centroService;
    protected $dataSourceName = "centcustfil_centcust_filial";
    protected $parameterBagColumns = ["NRORG", "CDFILIAL"];

    public function __construct($crudService, ParameterBag $parameterBag, Ger06300CentroDeCustoService $centroService, EntityManager $entityManager) {
        $this->centroService = $centroService;
        $this->entityManager    = $entityManager;
        parent::__construct($crudService, $parameterBag);
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataset()->getRows()[0];
            //$this->centroService->validatePadraoOnSave($row);
            if ($row['__is_new']){
                parent::save($request, $response);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        try {
            //$this->centroService->validatePadraoOnDelete($request->getRow());
            parent::delete($request, $response);
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function cadastroAutomaticoCentroCusto(DTO\Request $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
                
            $row        = $request->getRow();
            $this->centroService->cadastroAutomaticoCentroCusto($row);
            
            $this->entityManager->commit();
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}