<?php
namespace Teknisa\CADCTB\Factories;


use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb27200RegistroAjustePeriodoBaseFactory {

    protected $connection;
    protected $entityManager;
    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    public function acmExcluiLancContLalur($params){
        $sql = "DELETE
                     FROM LANCLALURA L
                     WHERE EXISTS (SELECT 1
                                    FROM CONTASLALURA C
                                    WHERE L.NRSEQCONTLALURA = C.NRSEQCONTLALURA
                                    AND C.NRSEQLALURA = :P_NRSEQLALURA)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function acmExcluiContasLalur($params){
        $sql = "DELETE
                     FROM CONTASLALURA
                     WHERE NRSEQLALURA = :P_NRSEQLALURA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function acmInsertLancParteA($row){
      
      
        $sql = "INSERT INTO LALURPARTEA
               (CDEMPRESA, DTLANCAMENT, CDLANCAMENT, IDRELACAO,
                DSHISTLANC, VRLANCAMENT, CDCONTCTBLB, IDTRIBUTO, NRSEQLALURA)
                VALUES
               (:CDEMPRESA, :DTLANCAMENT, :CDLANCAMENT, :IDRELACAO,
                :DSHISTLANC, :VRLANCAMENT, :CDCONTCTBLB, :IDTRIBUTO, :NRSEQLALURA)";
                
                
                
        $params = array(
            'CDEMPRESA'=>$row['CDEMPRESA'],
            'DTLANCAMENT'=>$row['DTLANCAMENT'],
            'CDLANCAMENT'=>$row['CDLANCAMENT'],
            'IDRELACAO'=>$row['IDRELACAO'],
            'DSHISTLANC'=>$row['DSHISTLANC'],
            'VRLANCAMENT'=>$row['VRLANCAMENT'],
            'CDCONTCTBLB'=> isset($row['CDCONTCTBLB']) ? $row['CDCONTCTBLB'] : null ,
            'IDTRIBUTO'=>$row['IDTRIBUTO'],
            'NRSEQLALURA'=>$row['NRSEQLALURA']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
    }
    
    public function getAdqDadosParteB($params) {
        $sql = "SELECT
                    L.CDEMPRESA,
                    L.CDCONTCTBL,
                    L.DTLANCAMENT,
                    L.DSHISTLANC,
                    L.VRDEBITO,
                    L.VRCREDITO,
                    L.NRSEQLALURB,
                    L.DTLIMITEUSO,
                    L.IDTRIBUTO,
                    L.NRSEQLALURA
                FROM
                    LALURPARTEB L
                WHERE
                    L.CDEMPRESA =:P_CDEMPRESA
                    AND L.NRSEQLALURA =:P_NRSEQLALURA";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch();
    }

    public function getAdqDadosParteA($params) {
        $sql = "SELECT
                    L.NRSEQLALURA,
                    L.CDEMPRESA,
                    L.DTLANCAMENT,
                    L.CDLANCAMENT,
                    M.DSLANCAMENT,
                    M.IDTIPOLANC,
                    L.IDRELACAO,
                    L.DSHISTLANC,
                    L.VRLANCAMENT,
                    L.CDCONTCTBLB,
                    L.IDTRIBUTO,
                    L.NRORG
                FROM
                    LALURPARTEA L,
                    M300A_LALURA M
                WHERE
                    L.CDEMPRESA =:P_CDEMPRESA
                    AND L.NRSEQLALURA =:P_NRSEQLALURA
                    AND L.CDLANCAMENT = M.CDLANCAMENT (+)";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch();
    }
    
    public function getAcmDelLalurParteB($row) {
        $sql = "DELETE FROM LALURPARTEB WHERE CDEMPRESA = :P_CDEMPRESA AND NRSEQLALURA = :P_NRSEQLALURA";

        $params = array("P_CDEMPRESA" => $row['CDEMPRESA'],
                        "P_NRSEQLALURA" => $row['NRSEQLALURA']);
    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getAcmInsertLalurParteB($adqDadosParteA, $pVRDEBITO, $pVRCREDITO, $row, $adqDadosParteB) {
        $sql = "INSERT INTO LALURPARTEB (
                    CDEMPRESA,
                    CDCONTCTBL,
                    DTLANCAMENT,
                    DSHISTLANC,
                    VRDEBITO,
                    VRCREDITO,
                    NRSEQLALURB,
                    DTLIMITEUSO,
                    IDTRIBUTO,
                    NRSEQLALURA
                ) VALUES (
                    :P_CDEMPRESA,
                    :P_CDCONTCTBL,
                    :P_DTLANCAMENT,
                    :P_DSHISTLANC,
                    :P_VRDEBITO,
                    :P_VRCREDITO,
                    :P_NRSEQLALURB,
                    :P_DTLIMITEUSO,
                    :P_IDTRIBUTO,
                    :P_NRSEQLALURA
                )";

        $params = array("P_CDEMPRESA" => $adqDadosParteA['CDEMPRESA'],
                        "P_CDCONTCTBL" => $row['CDCONTCTBL'],
                        "P_DTLANCAMENT" => $adqDadosParteA['DTLANCAMENT'],
                        "P_DSHISTLANC" => $adqDadosParteA['DSHISTLANC'],
                        "P_VRDEBITO" => $pVRDEBITO,
                        "P_VRCREDITO" => $pVRCREDITO,
                        "P_NRSEQLALURB" => isset($row['NRSEQLALURB']) ? $row['NRSEQLALURB'] : $adqDadosParteB['NRSEQLALURB'],
                        "P_DTLIMITEUSO" => isset($adqDadosParteB['DTLIMITEUSO']) ? $adqDadosParteB['DTLIMITEUSO'] : null ,
                        "P_IDTRIBUTO" => isset($row['IDTRIBUTO']) ? $row['IDTRIBUTO'] : $adqDadosParteB['IDTRIBUTO'],  
                        "P_NRSEQLALURA" => $adqDadosParteA['NRSEQLALURA']);
        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
        
    }
    
    public function execSaldoLalurParteB($cdempresa, $cdcontctbl){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'SALDO_LALUR_PARTEB');
        $procedure->addParam(new Param('P_CDEMPRESA',  Param::PARAM_INPUT, $cdempresa,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCONTCTBL', Param::PARAM_INPUT, $cdcontctbl, Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    
    public function getHistoricoLancamento($cdempresa, $dtlanccont, $cdcontctbl) {
        $sql = "SELECT
                    LC.CDEMPRESA,
                    LC.NRLANCCONT,
                    LC.NRSEQLANC,
                    LC.DSHISTLANC,
                    ABS(LC.VRLANCAMENT) VRLANCAMENT,
                    DECODE(LC.VRLANCAMENT,ABS(LC.VRLANCAMENT),'CR','DB') ID
                FROM
                    CONTCTBL CC,
                    LANCCONT LC
                WHERE
                    LC.CDEMPRESA =:P_CDEMPRESA
                    AND LC.DTLANCCONT =:P_DTLANCCONT
                    AND LC.CDCONTCTBL =:P_CDCONTCTBL
                    AND LC.CDCONTCTBL = CC.CDCONTCTBL
                    AND CC.IDCONTCCUS <> 'R'
                ORDER BY
                    LC.VRLANCAMENT";
                    
        $params = array("P_CDEMPRESA" => $cdempresa,
                        "P_DTLANCCONT" => $dtlanccont,
                        "P_CDCONTCTBL" => $cdcontctbl);
    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function InsertContasLalur($row) {
        $sql = "INSERT INTO CONTASLALURA (
                    CDEMPRESA,       
                    DTLANCAMENT,    
                    CDLANCAMENT,
                    CDCONTCTBL,
                    NRSEQLALURA,
                    NRSEQCONTLALURA,
                    VRCDCONTCTBL,
                    NRORG
                    )
                    VALUES (
                    :CDEMPRESA,       
                    :DTLANCAMENT,    
                    :CDLANCAMENT,
                    :CDCONTCTBL,
                    :NRSEQLALURA,
                    :NRSEQCONTLALURA,
                    :VRCDCONTCTBL,
                    :NRORG
                    )";
        $params = array('CDEMPRESA'=>$row['CDEMPRESA'],
                        'DTLANCAMENT'=>$row['DTLANCAMENT'],
                        'CDLANCAMENT'=>$row['CDLANCAMENT'],
                        'CDCONTCTBL'=>$row['CDCONTCTBL'],
                        'NRSEQLALURA'=>$row['NRSEQLALURA'],
                        'NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA'],
                        'VRCDCONTCTBL'=>$row['VRCDCONTCTBL'],
                        'NRORG'=>$row['NRORG']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function acmUpdateLancParteA($row) {
        $sql = "UPDATE LALURPARTEA
                   SET DTLANCAMENT = :DTLANCAMENT,
                       CDLANCAMENT = :CDLANCAMENT,
                       IDRELACAO = :IDRELACAO,
                       DSHISTLANC = :DSHISTLANC,
                       VRLANCAMENT = :VRLANCAMENT,
                       CDCONTCTBLB = :CDCONTCTBLB
                 WHERE NRSEQLALURA = :NRSEQLALURA 
                        AND NRORG = :NRORG";
                 
        $params = array('DTLANCAMENT'=>$row['DTLANCAMENT'],
                        'CDLANCAMENT'=>$row['CDLANCAMENT'],
                        'IDRELACAO'=>$row['IDRELACAO'],
                        'DSHISTLANC'=>$row['DSHISTLANC'],
                        'VRLANCAMENT'=>$row['VRLANCAMENT'],
                        'CDCONTCTBLB'=>$row['CDCONTCTBLB'],
                        'NRSEQLALURA'=>$row['NRSEQLALURA'],
                        'NRORG'=>$row['NRORG']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    public function acmUpdateContasLancParteA($row) {
        
        $sql = "UPDATE CONTASLALURA
                   SET CDCONTCTBL = :CDCONTCTBL,
                       VRCDCONTCTBL = :VRCDCONTCTBL
                 WHERE NRSEQCONTLALURA = :NRSEQCONTLALURA
                        AND NRORG = :NRORG";
                 
        $params = array('CDCONTCTBL'=>$row['CDCONTCTBL'],
                        'VRCDCONTCTBL'=>$row['VRCDCONTCTBL'],
                        'NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA'],
                        'NRORG'=>$row['NRORG']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    
    public function getNrseqLanc($row) {
        
        $sql = "SELECT MAX(NRSEQLANC) AS NRSEQLANC 
                    FROM LANCLALURA
                    WHERE CDEMPRESA = :CDEMPRESA";
                 
        $params = array(
            'CDEMPRESA'=>$row['CDEMPRESA']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return (sprintf("%05d", ($std->fetch()['NRSEQLANC']+1)));
        
    }
    
    public function getNrseqLalurb($row) {
        
        $sql = "SELECT MAX(NRSEQLALURB) AS NRSEQLALURB 
                    FROM LALURPARTEB
                    WHERE CDEMPRESA = :CDEMPRESA";
                 
        $params = array(
            'CDEMPRESA'=>$row['CDEMPRESA']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return intval((sprintf("%05d", ($std->fetch()['NRSEQLALURB']+1))));
        
    }
    
    public function getNrseqLanclalura() {
        $sql = "SELECT MAX(NRSEQLANCLALURA) AS NRSEQLANCLALURA 
                    FROM LANCLALURA";
                 
        $params = array();
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return strval(($std->fetch()['NRSEQLANCLALURA']+1));

    }

    public function adqContasLancParteA($nrseqlalura){

        $sql = "SELECT L.CDEMPRESA, L.DTLANCAMENT, L.CDLANCAMENT, C.CDCONTCTBL, CC.NMCONTCTBL,
                   C.VRCDCONTCTBL, DECODE(C.VRCDCONTCTBL, ABS(C.VRCDCONTCTBL), 'C', 'D') ID,
                   C.NRSEQLALURA, C.NRSEQCONTLALURA
              FROM CONTASLALURA C, CONTCTBL CC, LALURPARTEA L
             WHERE C.NRSEQLALURA = :P_NRSEQLALURA
               AND C.NRSEQLALURA = L.NRSEQLALURA
               AND C.CDCONTCTBL = CC.CDCONTCTBL
             ORDER BY C.CDCONTCTBL";
            
        $params = array(
            'P_NRSEQLALURA'=>$nrseqlalura
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        
        return $std->fetch();
    }


    public function adqLancamentosContas($params){

        $sql = "SELECT L.CDEMPRESA, C.CDCONTCTBL, LC.DTLANCCONT, LC.NRLANCCONT, LC.NRSEQLANC, LC.DSHISTLANC,
                   ABS(LC.VRLANCAMENT) VRLANCAMENT, DECODE(LC.VRLANCAMENT, ABS(LC.VRLANCAMENT), 'CR', 'DB') ID,
                   L.NRSEQCONTLALURA, L.NRSEQLANCLALURA
              FROM CONTASLALURA C, CONTCTBL CC, LANCLALURA L, LANCCONT LC, LALURPARTEA LA
             WHERE C.NRSEQLALURA = :P_NRSEQLALURA
               AND ((C.CDCONTCTBL = :P_CDCONTCTBL) OR ('T' = :P_CDCONTCTBL))
               AND C.CDCONTCTBL = CC.CDCONTCTBL
               AND C.NRSEQLALURA = LA.NRSEQLALURA
               AND C.NRSEQCONTLALURA = L.NRSEQCONTLALURA
               AND L.NRLANCCONT = LC.NRLANCCONT
               AND C.CDCONTCTBL = LC.CDCONTCTBL
               AND L.NRSEQLANC = LC.NRSEQLANC
               AND LA.CDEMPRESA = LC.CDEMPRESA
               AND CC.IDCONTCCUS <> 'R'
             ORDER BY LC.VRLANCAMENT";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function acmExcluiLancamentosConta($row){

        $sql = "DELETE
                  FROM LANCLALURA
                 WHERE NRSEQCONTLALURA = :P_NRSEQCONTLALURA";
            
        $params = array(
            'P_NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function acmDeleteContasLancParteA($row){

        $sql = "DELETE
                  FROM CONTASLALURA
                 WHERE NRSEQCONTLALURA = :NRSEQCONTLALURA";
            
        $params = array(
            'NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function insertLancContasAuto ($row){

        $sql = "SELECT LC.CDEMPRESA, LC.CDCONTCTBL, LC.DTLANCCONT, LC.NRLANCCONT, LC.NRSEQLANC, LC.DSHISTLANC,
                       ABS(LC.VRLANCAMENT) VRLANCAMENT, DECODE(LC.VRLANCAMENT, ABS(LC.VRLANCAMENT), 'CR', 'DB') ID, 
                       NRORG, VRLANCAMENT AS VALOR
                  FROM LANCCONT LC
                 WHERE LC.CDEMPRESA = :CDEMPRESA
                   AND LC.DTLANCCONT BETWEEN :DTINICIAL AND :DTFINAL
                   AND LC.CDCONTCTBL = :CDCONTCTBL
                   AND LC.IDLANCINTE <> 'F'
             ORDER BY LC.VRLANCAMENT";
        $params = array(
            'CDEMPRESA'=>$row['CDEMPRESA'],
            'DTINICIAL'=>$row['DTINICIAL'],
            'DTFINAL'=>$row['DTFINAL'],
            'CDCONTCTBL'=>$row['CDCONTCTBL']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $rowsToInsert = $std->fetchAll();
        $VALORFINAL = 0;
        foreach($rowsToInsert as $key => $rowToInsert) {
            $rowToInsert['NRSEQCONTLALURA'] = $row['NRSEQCONTLALURA'];
            $rowToInsert['NRSEQLANCLALURA'] = $this->getNewNRSEQLANCLALURA();
            $this->acmInsertLancContas($rowToInsert);
            $VALORFINAL +=  $rowToInsert['VALOR'];
        }
        
        return $VALORFINAL;
    }
    
    public function getNewNRSEQLANCLALURA (){
        $NRSEQLANCLALURA = $this->connection->prepare('SELECT MAX(NRSEQLANCLALURA)+1 AS NRSEQLANCLALURA FROM LANCLALURA');
        $NRSEQLANCLALURA->execute([]);
        return $NRSEQLANCLALURA->fetchAll()[0]['NRSEQLANCLALURA'];
    }
    public function acmInsertLancContas ($row){
        
        $params = array(
            'CDEMPRESA'=>$row['CDEMPRESA'],
            'NRLANCCONT'=>$row['NRLANCCONT'],
            'NRSEQLANC'=>$row['NRSEQLANC'],
            'NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA'],
            'NRSEQLANCLALURA'=>$row['NRSEQLANCLALURA']
        );
        
        $sqlInsert = "INSERT INTO LANCLALURA
                       (CDEMPRESA, NRLANCCONT, NRSEQLANC, NRSEQCONTLALURA, NRSEQLANCLALURA)
                    VALUES
                       (:CDEMPRESA, :NRLANCCONT, :NRSEQLANC, :NRSEQCONTLALURA, :NRSEQLANCLALURA)";
        
        $std = $this->connection->prepare($sqlInsert);
        $std->execute($params);
        
        return true;
    } 

    public function getContaByNRSEQCONTLALURA ($row){
        $sql = "SELECT *
                  FROM CONTASLALURA 
                 WHERE NRSEQCONTLALURA = :NRSEQCONTLALURA
                ";
        $params = array(
            'NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll()[0];
    }
    
    public function deleteLancContabeisEnvolvidos($row){
        $sql = "DELETE
                  FROM LANCLALURA
                 WHERE NRSEQCONTLALURA = :NRSEQCONTLALURA
                   AND NRSEQLANCLALURA = :NRSEQLANCLALURA";
            
        $params = array(
            'NRSEQCONTLALURA'=>$row['NRSEQCONTLALURA'],
            'NRSEQLANCLALURA'=>$row['NRSEQLANCLALURA']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    public function deleteTabelaImportada($row){
        $sqlDelete = "DELETE FROM ".$row['TABLE'];
       
        $std = $this->connection->prepare($sqlDelete);
        $std->execute();
    }
    
    public function salvarTabelaImportada($row){

        $sql = "INSERT INTO  ".$row['TABLE']."
           (CDLANCAMENT, DSLANCAMENT, DTINICIO, DTFIM, DSFORMULA, IDTIPOLANC)
        VALUES
           (:CDLANCAMENT, :DSLANCAMENT, :DTINICIO, :DTFIM, :DSFORMULA, :IDTIPOLANC)";
    
        $params = array(
            'CDLANCAMENT'    => $row['CDLANCAMENT'],
            'DSLANCAMENT'    => $row['DSLANCAMENT'],
            'DTINICIO'       => $row['DTINICIO']   ,
            'DTFIM'          => $row['DTFIM']      ,
            'DSFORMULA'      => $row['DSFORMULA']  ,
            'IDTIPOLANC'     => $row['IDTIPOLANC'] 
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

}